package com.ningdatech.pmapi.projectlib.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.ValidUtil;
import com.ningdatech.pmapi.common.constant.CommonConst;
import com.ningdatech.pmapi.common.enumeration.CommonEnum;
import com.ningdatech.pmapi.common.helper.UserInfoHelper;
import com.ningdatech.pmapi.common.model.entity.ExcelExportWriter;
import com.ningdatech.pmapi.common.statemachine.util.StateMachineUtils;
import com.ningdatech.pmapi.common.util.ExcelDownUtil;
import com.ningdatech.pmapi.datascope.model.DataScopeDTO;
import com.ningdatech.pmapi.datascope.utils.DataScopeUtil;
import com.ningdatech.pmapi.projectlib.constant.ImportTemplateConstant;
import com.ningdatech.pmapi.projectlib.enumeration.ImportTemplateEnum;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectStatusEnum;
import com.ningdatech.pmapi.projectlib.helper.ProjectHelper;
import com.ningdatech.pmapi.projectlib.model.dto.AnnualLibImportDTO;
import com.ningdatech.pmapi.projectlib.model.dto.ProjectDTO;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.model.req.ProjectApprovedReq;
import com.ningdatech.pmapi.projectlib.model.req.ProjectIdReq;
import com.ningdatech.pmapi.projectlib.model.req.ProjectListReq;
import com.ningdatech.pmapi.projectlib.model.req.StartProjectDeclareReq;
import com.ningdatech.pmapi.projectlib.model.vo.AnnualPlanListItemVO;
import com.ningdatech.pmapi.projectlib.service.INdProjectStatusChangeService;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.ningdatech.pmapi.user.security.auth.model.UserFullInfoDTO;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.util.*;

import static com.ningdatech.pmapi.projectlib.enumeration.ProjectStatusEnum.*;

/**
 * <p>
 * AnnualPlanLibManage
 * </p>
 *
 * @author WendyYang
 * @since 14:09 2023/2/11
 */
@Component
@AllArgsConstructor
@Slf4j
public class AnnualPlanLibManage {

    private final IProjectService projectService;
    private final StateMachineUtils stateMachine;
    private final INdProjectStatusChangeService statusChangeService;
    private final UserInfoHelper userInfoHelper;

    /**
     * 年度计划查询状态
     */
    private static final List<ProjectStatusEnum> ANNUAL_PLAN_LIST_STATUS = Arrays.asList(
            IN_THE_ANNUAL_PLAN,
            SCHEME_UNDER_REVIEW,
            SCHEME_REVIEW_FAILED,
            TO_BE_APPROVED,
            TO_BE_DECLARED,
            PLAN_TO_BE_DECLARED,
            PENDING_PREQUALIFICATION_CHOICE,
            PROJECT_APPROVED,
            TO_BE_PURCHASED,
            UNDER_CONSTRUCTION,
            TO_BE_FINALLY_INSPECTED,
            FINAL_ACCEPTANCE_IS_UNDER_REVIEW,
            FINAL_ACCEPTANCE_REVIEW_FAILED,
            ARCHIVED
    );


    public PageVo<AnnualPlanListItemVO> annulPlanLibList(ProjectListReq req) {
        LambdaQueryWrapper<Project> query = ProjectHelper.projectQuery(req);
        Integer isTemporaryAugment = req.getIsTemporaryAugment();
        if (Objects.isNull(isTemporaryAugment)) {
            throw new BizException("请传入是否临时增补标志!");
        }
        query.eq(Project::getIsTemporaryAugment, isTemporaryAugment);
        query.orderByDesc(Project::getAnnualPlanAddTime);
        query.in(Project::getStatus, CollUtils.fieldList(ANNUAL_PLAN_LIST_STATUS, ProjectStatusEnum::getCode));
        //数据权限
        buildProjectLibPermission(query);
        Page<Project> page = projectService.page(req.page(), query);
        long total;
        if ((total = page.getTotal()) == 0) {
            return PageVo.empty();
        }
        PageVo<AnnualPlanListItemVO> result = PageVo.of(new ArrayList<>(), total);
        page.getRecords().forEach(w -> {
            AnnualPlanListItemVO item = new AnnualPlanListItemVO();
            item.setProjectId(w.getId());
            item.setProjectId(w.getId());
            item.setProjectName(w.getProjectName());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectYear(w.getProjectYear());
            item.setProjectType(w.getProjectType());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setBuildOrg(w.getBuildOrgName());
            item.setCreateOn(w.getCreateOn());
            item.setIsStartDeclaredProject(!IN_THE_ANNUAL_PLAN.eq(w.getStatus()));
            result.getRecords().add(item);
        });
        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    public void startProjectDeclared(StartProjectDeclareReq req) {
        List<Project> projects = projectService.listByIds(req.getProjectIds());
        projects.forEach(w -> {
            if (!IN_THE_ANNUAL_PLAN.eq(w.getStatus())) {
                throw BizException.wrap("开启方案申报失败");
            }
        });
        projects.forEach(project -> {
            try {
                stateMachine.pass(project);
            } catch (Exception e) {
                throw new BizException("开启方案申报失败");
            }
        });
        projectService.updateBatchById(projects);
    }

    @Transactional(rollbackFor = Exception.class)
    public void projectApproved(ProjectApprovedReq req) {
        Project project = projectService.getById(req.getProjectId());
        stateMachine.pass(project);
        LambdaUpdateWrapper<Project> update = Wrappers.lambdaUpdate(Project.class)
                .set(Project::getApprovalAmount, req.getApprovedAmount())
                .set(Project::getApprovedFile, req.getApprovedFileId())
                .set(Project::getBuildCycle, req.getBuildCycle())
                .set(Project::getApprovedConstructionPlanFile, req.getBuildPlanFileId())
                .set(Project::getApprovalDate, req.getApprovedDate())
                .set(Project::getStatus, project.getStatus())
                .set(Project::getStage, project.getStage())
                .eq(Project::getId, req.getProjectId());
        projectService.update(update);
    }

    @Transactional(rollbackFor = Exception.class)
    public void suspendAnnualPlan(ProjectIdReq req) {
        Project project = projectService.getById(req.getProjectId());
        stateMachine.reject(project);
        projectService.updateById(project);
    }

    @Transactional(rollbackFor = Exception.class)
    public void importAnnualPlan(ImportTemplateEnum template, MultipartFile file) {
        String contentType = file.getContentType();
        if (!contentType.equals(ExcelUtil.XLS_CONTENT_TYPE) &&
                !contentType.equals(ExcelUtil.XLSX_CONTENT_TYPE)
        ) {
            throw BizException.wrap("导入失败,不支持的文件类型,请按照提供的模板导入文件!");
        }
        try (InputStream inputStream = file.getInputStream();
             ExcelReader reader = ExcelUtil.getReader(inputStream)) {
            Map<String, String> alias;
            List<String> title = ImportTemplateConstant.getTemplateTitle(template);
            switch (template) {
                case ANNUAL_PLAN:
                    alias = new HashMap<>(title.size());
                    alias.put(title.get(0), "id");
                    alias.put(title.get(1), "projectId");
                    alias.put(title.get(2), "projectName");
                    alias.put(title.get(3), "projectIntroduction");
                    alias.put(title.get(4), "buildBasis");
                    alias.put(title.get(5), "isFirst");
                    alias.put(title.get(6), "buildCycle");
                    alias.put(title.get(7), "declaredAmount");
                    alias.put(title.get(8), "annualPlanAmount");
                    alias.put(title.get(9), "declareHaveAmount");
                    alias.put(title.get(10), "declareGovOwnFinanceAmount");
                    alias.put(title.get(11), "declareGovSuperiorFinanceAmount");
                    alias.put(title.get(12), "declareBankLendingAmount");
                    alias.put(title.get(13), "declareOtherAmount");
                    alias.put(title.get(14), "firstQuarter");
                    alias.put(title.get(15), "secondQuarter");
                    alias.put(title.get(16), "thirdQuarter");
                    alias.put(title.get(17), "fourthQuarter");
                    alias.put(title.get(18), "buildUnitName");
                    alias.put(title.get(19), "contactName");
                    alias.put(title.get(20), "responsibleMan");
                    alias.put(title.get(21), "projectRemarks");
                    reader.setHeaderAlias(alias);
                    importAnnualPlanData(reader.readAll(AnnualLibImportDTO.class));
                    break;
                default:
                    throw new BizException("不支持的数据导入类型");
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private void importAnnualPlanData(List<AnnualLibImportDTO> importDataList) {
        if (CollectionUtils.isEmpty(importDataList)) {
            return;
        }
        List<Project> projectList = new ArrayList<>();
        List<Long> projectIds = CollUtils.fieldList(importDataList, AnnualLibImportDTO::getProjectId);
        Assert.isTrue(projectIds.size() == importDataList.size(), "项目ID不可重复");
        Long userId = LoginUserUtil.getUserId();
        LocalDateTime now = LocalDateTime.now();
        importDataList.forEach(w -> {
            Project project = new Project();
            project.setCreateBy(userId);
            project.setCreateOn(now);
            project.setUpdateBy(userId);
            project.setUpdateOn(now);

            project.setId(w.getProjectId());
            project.setProjectName(w.getProjectName());
            project.setProjectIntroduction(w.getProjectIntroduction());
            // 建设依据(立项依据忽略)
            if (CommonConst.NEW_CONSTRUCTION.equals(w.getIsFirst())){
                project.setIsFirst(CommonEnum.YES.getCode());
            }else if (CommonConst.CONTINUED_CONSTRUCTION.equals(w.getIsFirst())){
                project.setIsFirst(CommonEnum.NO.getCode());
            }
            String buildCycle = w.getBuildCycle();
            int index = buildCycle.indexOf(CommonConst.MONTH);
            if (-1 == index){
                throw new BizException("项目ID为:"  + w.getProjectId() +  "的建设起止年限格式不正确,请按照xx年xx月至xx年xx月的格式输入!");
            }
            String beginTime = buildCycle.substring(0, index + 1);
            project.setBeginTime(beginTime);
            String endTime = buildCycle.substring(index + 2);
            project.setEndTime(endTime);
            project.setDeclareAmount(w.getDeclaredAmount());
            project.setAnnualPlanAmount(w.getAnnualPlanAmount());
            project.setDeclareHaveAmount(w.getDeclareHaveAmount());
            project.setDeclareGovOwnFinanceAmount(w.getDeclareGovOwnFinanceAmount());
            project.setDeclareGovSuperiorFinanceAmount(w.getDeclareGovSuperiorFinanceAmount());
            project.setDeclareBankLendingAmount(w.getDeclareBankLendingAmount());
            project.setDeclareOtherAmount(w.getDeclareOtherAmount());
            project.setEngineeringSpeedOne(w.getFirstQuarter());
            project.setEngineeringSpeedTwo(w.getSecondQuarter());
            project.setEngineeringSpeedThree(w.getThirdQuarter());
            project.setEngineeringSpeedFour(w.getFourthQuarter());

            project.setBuildOrgName(w.getBuildUnitName());
            project.setContactName(w.getContactName());
            project.setResponsibleMan(w.getResponsibleMan());
            project.setProjectRemarks(w.getProjectRemarks());
            projectList.add(project);
        });
        LambdaQueryWrapper<Project> delQuery = Wrappers.lambdaQuery(Project.class)
                .in(Project::getId, projectIds);
        projectService.remove(delQuery);
        projectService.saveBatch(projectList);
    }

    public void updateAnnualPlan(ProjectDTO req) {
        Project project = BeanUtil.copyProperties(req, Project.class);
        projectService.updateById(project);
    }

    public void exportList(ProjectListReq param, HttpServletResponse response) {
        param.setPageNumber(CommonConst.EXPORT_PAGE_NUMBER);
        param.setPageSize(CommonConst.EXPORT_PAGE_SIZE);
        Integer isTemporaryAugment = param.getIsTemporaryAugment();
        if (Objects.isNull(isTemporaryAugment)) {
            throw new BizException("请传入是否临时增补标志!");
        }
        param.setIsTemporaryAugment(isTemporaryAugment);
        param.setStatusList(CollUtils.fieldList(ANNUAL_PLAN_LIST_STATUS, ProjectStatusEnum::getCode));
        LambdaQueryWrapper<Project> query = ProjectHelper.projectQuery(param);
        query.orderByDesc(Project::getAnnualPlanAddTime);
        List<Project> projects = projectService.list(query);

        ExcelExportWriter excelExportWriter = new ExcelExportWriter();

        Integer tableFlag = param.getTableFlag();
        if (Objects.isNull(tableFlag)) {
            throw new BizException("请传入要导出的表格类型!");
        }
        String fileName = null;
        if (tableFlag == 0) {
            fileName = "年度计划库编辑表";
        } else if (tableFlag == 1) {
            fileName = "年度计划库列表";
        }
        excelExportWriter.setFileName(fileName);
        List<String> sheetsNames = new ArrayList<>();
        sheetsNames.add(fileName);
        // 表体行数据集合
        List<List<String>> exportDatas = ExcelDownUtil.getProjectExportDatas(param.getExportOptionList(), projects);
        // sheet列表集合
        List<List<List<String>>> sheets = new ArrayList<>();
        sheets.add(exportDatas);
        excelExportWriter.setHeads(sheets);
        excelExportWriter.setDatas(exportDatas);
        excelExportWriter.setSheets(sheetsNames);
        excelExportWriter.setHeadPropertyClass(List.class);
        excelExportWriter.setHeadPropertyFlag(Boolean.FALSE);
        ExcelDownUtil.excelExportFile(response, excelExportWriter);
    }

    /**
     * build 项目的角色权限到req
     *
     * @param query
     */
    public UserFullInfoDTO buildProjectLibPermission(LambdaQueryWrapper<Project> query) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        Optional<DataScopeDTO> currentUserDataScope = DataScopeUtil.getCurrentUserDataScope(user);
        if (!currentUserDataScope.isPresent()) {
            log.warn("没有取到权限信息 当前查询 没有权限条件");
            return user;
        }
        switch (currentUserDataScope.get().getRole()) {
            case NORMAL_MEMBER:
                //普通用户 只能看到自己单位去申报的
                query.eq(Project::getBuildOrgCode, user.getEmpPosUnitCode());
                break;
            case COMPANY_MANAGER:
                //单位管理员 看到自己单位去申报的 + 待预审的主管单位是自己单位的项目
                query.eq(Project::getBuildOrgCode, user.getEmpPosUnitCode());
                break;
            case SUPER_ADMIN:
                //超级管理员 看到丽水全市的 并且也要判断他 同时是不是单位管理员
                break;
            case REGION_MANAGER:
                //区域管理员 看到自己区域的项目
                query.eq(Project::getAreaCode, user.getRegionCode());
                break;
            default:
                //没有权限的话 就让它查不到
                query.eq(Project::getId, "NULL");
                break;
        }
        return user;
    }
}
